#lang racket/gui

;;; more-buttons-simple.rkt  : a circle is a list of 4 elements !
;;; PF2 Autumn 2016 - simple version : circles do not move !
;;; Racket 6.6

; a circle is represented by a list (x y radius num-color), or better a struct or an object...

(define LCERCLES '())
(define NBCERCLES (length LCERCLES))
(define BRUSHES      ; a pen for drawing and a brush for painting (filling) !
  (list (make-object brush% "white" 'solid)
        (make-object brush% "red" 'solid)
        (make-object brush% "blue" 'solid)
        (make-object brush% "black" 'solid)))
(define SIZE 300)    ; canvas size

(define FRAME 
  (new frame% 
       (label "More buttons...") 
       (stretchable-width #f) 
       (stretchable-height #f)))

(define HPANEL 
  (new horizontal-panel% (parent FRAME)))

(define VPANEL 
  (new vertical-panel% (parent HPANEL)))

(define CANVAS 
  (new canvas% 
       (parent HPANEL) 
       (style '(border))
       (min-width SIZE)
       (min-height SIZE)
       (paint-callback (lambda (c dc)
                         (send dc clear)
                         (for ([pt (in-list LCERCLES)])
                           (match-define (list x y r nc) pt)
                           ; match-define like in O'CAML... Cool. See at the bootom of this page
                           ; if you don't like, uncomment the next line :
                           ;(let ((x (car pt)) (y (cadr pt)) (r (caddr pt)) (nc (cadddr pt))) 
                           (send (send CANVAS get-dc) set-brush (list-ref BRUSHES nc)) 
                           (send dc draw-ellipse (- x r) (- y r) (* 2 r) (* 2 r)))))))

(define GAUGE 
  (new gauge% 
       (parent VPANEL) 
       (label "nbPoints") 
       (range 14)))    ; in [0,14] ==> 15 points max !

(define BUTTON1 
  (new button% (parent VPANEL) (label "New")
       (callback (lambda (obj evt)
                   (define x (random SIZE)) 
                   (define y (random SIZE))
                   (define radius (send SLIDER get-value))
                   (define num-color (send RADIO get-selection))
                   (set! LCERCLES (cons (list x y radius num-color) LCERCLES))
                   (set! NBCERCLES (+ NBCERCLES 1))
                   (send GAUGE set-value (+ 1 (send GAUGE get-value)))
                   (when (= NBCERCLES (send GAUGE get-range))
                     (send obj enable #f))
                   (send CANVAS refresh-now)))))       ; immediately post a paint query

(define BUTTON2 
  (new button% 
       (parent VPANEL) 
       (label "Clear")
       (callback (lambda (b evt)
                   (set! LCERCLES '())
                   (set! NBCERCLES 0)
                   (send (send CANVAS get-dc) clear)   ; nevers define the dc as a global variable !
                   (send GAUGE set-value 0)
                   (send BUTTON1 enable #t)))))

(define RADIO 
  (new radio-box% 
       (parent VPANEL) 
       (label "Local color")
       (choices '("white" "red" "blue" "black"))
       (callback (lambda (r evt)
                   (define n (send r get-selection))
                   (send (send CANVAS get-dc) set-brush (list-ref BRUSHES n))
                   (send CANVAS refresh)))))

(define SLIDER 
  (new slider% 
       (parent VPANEL) 
       (label "Size") 
       (style '(vertical))
       (min-value 5) 
       (max-value 30) 
       (init-value 10)))
       ;(callback void)))   ; optional here, no immediate action...

(send FRAME show #t)
