;;; more-buttons-sol.rkt (avec la souris)
;;; Racket 6.6
;;; PF2 Automne 2016
;;; Ceci n'est pas une extension de la version 1, mais une nouvelle version.

#lang racket/gui

; les variables globales

(define LCERCLES '())    ; la liste de tous les cercles (des objets)
(define NBCERCLES 0)     ; pour ne pas recalculer la longueur chaque fois...

; the-brush-list is created automagically as an instance of brush-list%
; http://docs.racket-lang.org/draw/brush-list_.html

; Un cercle est un objet de la classe cercle%
; J'ai fait une classe cercle% comme entrainement. Une struct suffisait,
; mais l'objet ca fait class :-) Mmmmm...

(define cercle%
  (class object%
    (init-field (x 0) (y 0) (r 0) (color "black"))  
    (define/public (getX) x)
    (define/public (getY) y)
    (define/public (getR) r)
    (define/public (getColor) color)
    (define/public (setX! nx) (set! x nx))
    (define/public (setY! ny) (set! y ny))
    (define/public (draw-with the-current-dc)
      (send the-current-dc set-brush (send the-brush-list find-or-create-brush color 'solid))
      (send the-current-dc draw-ellipse (- x r) (- y r) (* 2 r) (* 2 r)))
    (define/public (toString)
      (format "<~a,~a,~a,~a>" (getX) (getY) (getR) (getColor)))
    (super-new)
    (set! LCERCLES (cons this LCERCLES))
    (set! NBCERCLES (add1 NBCERCLES))
    (send GAUGE set-value (+ 1 (send GAUGE get-value)))
    (when (>= NBCERCLES (send GAUGE get-range))
      (send BUTTON1 enable #f))))

; ****************** GUI ************************************

(define SIZE 300)          ; taille du canvas

(define FRAME 
  (new frame% 
       (label "More buttons...") 
       (stretchable-width #f) 
       (stretchable-height #f)))

(define HPANEL
  (new horizontal-panel% (parent FRAME)))

(define VPANEL 
  (new vertical-panel%  (parent HPANEL)))

(define (trouve-cercle x y L)
  (define (dans-cercle? x y c)
    (<= (+ (sqr (- x (send c getX))) (sqr (- y (send c getY)))) (sqr (send c getR))))
  (cond
    ((empty? L) #f)
    ((dans-cercle? x y (car L)) (car L))
    (else (trouve-cercle x y (cdr L)))))

(define MY-CANVAS%
  (class canvas%               ; une sous-classe de canvas% qui gere la souris
    (define V #f)              ; le cercle courant choisi par la souris. J'imite le code du cours 6
    (define delta-x 0)
    (define delta-y 0)
    (define/override (on-event evt)    ; pour gerer la souris (pour le clavier, redefinir on-char)
      (case (send evt get-event-type) 
        ((left-down) (define x (send evt get-x)) 
                     (define y (send evt get-y))
                     (set! V (trouve-cercle x y LCERCLES))
                     (when V
                       (set! delta-x (- (send V getX) x))
                       (set! delta-y (- (send V getY) y))))
        ((left-up) (set! V #f))
        (else (when V
                (send V setX! (+ (send evt get-x) delta-x))
                (send V setY! (+ (send evt get-y) delta-y))
                (send this refresh-now)))))
    (super-new)))

(define CANVAS 
  (new MY-CANVAS% 
       (parent HPANEL) 
       (style '(border))
       (min-width SIZE)
       (min-height SIZE)
       (paint-callback     ; action : tout redessiner (est active par la methode on-paint ou refresh-now)
        (lambda (c dc)
          (send dc clear)
          (for ([cercle (in-list (reverse LCERCLES))])   ; pour l'ordre, hum...
            (send cercle draw-with dc))))))

(define GAUGE 
  (new gauge% 
       (parent VPANEL) 
       (label "nbPoints") 
       (range 14)))

(define BUTTON1 
  (new button% 
       (parent VPANEL) 
       (label "New")
       (callback     ; action : construire un cercle, l'empiler dans LCERCLES, tout redessiner
        (lambda (b evt)
          (define x (random SIZE)) 
          (define y (random SIZE))
          (define radius (send SLIDER get-value))
          (define n (send RADIO get-selection))
          (make-object cercle% x y radius (send RADIO get-item-label n))
          (send CANVAS refresh-now)))))

(define BUTTON2 
  (new button% (parent VPANEL) (label "Clear")
       (callback     ; action : effacer le canvas graphique
        (lambda (b evt)
          (set! LCERCLES '())
          (set! NBCERCLES 0)
          (send (send CANVAS get-dc) clear)
          (send GAUGE set-value 0)
          (send BUTTON1 enable #t)))))

(define RADIO 
  (new radio-box% 
       (parent VPANEL) 
       (label "Local color")
       (choices '("white" "red" "blue" "black"))
       (callback    ; action : demander au peintre de changer sa brosse, aucun effet sur le canvas
        (lambda (r evt)
          (define n (send r get-selection))
          (define color (send r get-item-label n))
          (send (send CANVAS get-dc) set-brush 
                (send the-brush-list find-or-create-brush color 'solid))))))

(define SLIDER 
  (new slider% 
       (parent VPANEL) 
       (label "Point size") 
       (style '(vertical))
       (min-value 5) 
       (max-value 30) 
       (init-value 10)))
       ;(callback void)))     ; optionnel, aucune action immediate

(send FRAME show #t)

